from django.test import TestCase

# Create your tests here.
from django.test.client import Client
from django.http import HttpResponse
from django.contrib.auth.models import User

import json
import uuid

class CreateTest(TestCase):
    def setUp(self):
        self.client = Client()

        self.url = "/api/tournaments/"

        self.username = str(uuid.uuid4())
        self.password = str(uuid.uuid4())

        self.nb_players_by_game = 2
        self.nb_players = 8

        user: User = User.objects.create_user(username=self.username, password=self.password)
        self.client.login(username=self.username, password=self.password)

    def test_normal(self):
        response: HttpResponse = self.client.post(self.url, {"nb_players_by_game": self.nb_players_by_game, "nb_players": self.nb_players})
        response_data: dict = json.loads(response.content)
        self.assertDictContainsSubset({"name": ""}, response_data)

    def test_too_small_nb_players_by_game(self):
        response: HttpResponse = self.client.post(self.url, {"nb_players_by_game": 1, "nb_players": self.nb_players})
        response_data = json.loads(response.content)
        self.assertDictEqual(response_data, {'nb_players_by_game': ['The numbers of players by game must be greather than 2.']})
    
    def test_too_small_nb_players(self):
        response: HttpResponse = self.client.post(self.url, {"nb_players_by_game": self.nb_players_by_game, "nb_players": 1})
        response_data = json.loads(response.content)
        self.assertDictEqual(response_data, {'nb_players': ['The numbers of players must be greather than 2.'], 'nb_players_by_game': ['The numbers of players by game must be smaller than the numbers of players.']})

    def test_nb_players_smaller_nb_players_by_game(self):
        response: HttpResponse = self.client.post(self.url, {"nb_players_by_game": 5, "nb_players": 3})
        response_data = json.loads(response.content)
        self.assertDictEqual(response_data, {'nb_players_by_game': ['The numbers of players by game must be smaller than the numbers of players.']})